/******************************************************************************
**                                                                        
** Source File Name = dropcli.c  %I%                                      
**                                                                        
** Licensed Materials - Property of IBM                                   
**                                                                        
** (C) COPYRIGHT International Business Machines Corp. 1999
** All Rights Reserved.                                                   
**                                                                        
** US Government Users Restricted Rights - Use, duplication or            
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      
**                                                                        
**                                                                        
**    PURPOSE :                                                           
** Drops all Tables, User Defined Types & Functions for the Order scenario.
**
**                                                                        
** For more information about these samples see the README file.
**
** For more information on programming in CLI see the:
**     - "Building CLI Applications" section of the Application Building Guide, and the
**     - CLI Guide and Reference.
**
** For more information on the SQL language see the SQL Reference.
**
********************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sqlcli1.h>
#include "samputil.h"          /* Header file for CLI sample code */

/* For the Macintosh environment when generating 68K applications */
#ifdef DB268K
   /* Need to include ASLM for 68K applications */
   #include <LibraryManager.h>
#endif

/*
 * Global Variables for user id and password.
 * To keep samples simple, not a recommended practice.
 */
extern SQLCHAR server[SQL_MAX_DSN_LENGTH + 1] ;
extern SQLCHAR uid[MAX_UID_LENGTH + 1] ;
extern SQLCHAR pwd[MAX_PWD_LENGTH + 1] ;

/* main */

int main( int argc, char * argv[] ) {

    SQLHANDLE henv, hdbc, hstmt ;
    SQLRETURN rc ;
    SQLINTEGER i ;

    /* Initialize SQL statement strings */
    SQLCHAR * stmt[] = {

        ( SQLCHAR * ) "DROP FUNCTION PRICE( CHAR(12), PUNIT, char(16) )",

        ( SQLCHAR * ) "DROP FUNCTION \"+\"( PNUM, INTEGER )",

        ( SQLCHAR * ) "DROP FUNCTION MAX( PNUM )",

        ( SQLCHAR * ) "DROP TABLE CUSTOMER",

        ( SQLCHAR * ) "DROP TABLE PRODUCT",

        ( SQLCHAR * ) "DROP TABLE PROD_PARTS",

        ( SQLCHAR * ) "DROP TABLE ORD_CUST",

        ( SQLCHAR * ) "DROP TABLE ORD_LINE",

        ( SQLCHAR * ) "DROP DISTINCT TYPE CNUM",

        ( SQLCHAR * ) "DROP DISTINCT TYPE PUNIT",

        ( SQLCHAR * ) "DROP DISTINCT TYPE UPRICE",

        ( SQLCHAR * ) "DROP DISTINCT TYPE PRICE",

        ( SQLCHAR * ) "DROP DISTINCT TYPE PNUM",

        ( SQLCHAR * ) "DROP DISTINCT TYPE ONUM",

        ( SQLCHAR * ) 0,

    } ;

    /* For the Macintosh environment when generating 68K applications */
    #ifdef DB268K
       /*
        Before making any API calls for 68K environment,
        need to initialize the Library Manager
       */
       InitLibraryManager(0,kCurrentZone,kNormalMemory);
       atexit(CleanupLibraryManager);
    #endif

    /* macro to initalize server, uid and pwd */
    INIT_UID_PWD ;

    /* allocate an environment handle    */
    rc = SQLAllocHandle( SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv ) ;
    if ( rc != SQL_SUCCESS ) return( terminate( henv, rc ) ) ;

    /* allocate a connect handle, and connect */
    rc = DBconnect( henv, &hdbc ) ;
    if ( rc != SQL_SUCCESS ) return( terminate( henv, rc ) ) ;

    rc = SQLAllocHandle( SQL_HANDLE_STMT, hdbc, &hstmt ) ;
    CHECK_HANDLE( SQL_HANDLE_DBC, hdbc, rc ) ;

    /* Execute Direct statements */
    i = 0 ;
    while ( stmt[i] != ( SQLCHAR * ) 0 ) {
       printf( ">Executing Statement %ld:\n", ( i + 1 ) ) ;
       printf( "    %s\n", stmt[i]);
       rc = SQLExecDirect( hstmt, stmt[i], SQL_NTS ) ;
       CHECK_HANDLE( SQL_HANDLE_STMT, hstmt, rc ) ;
       i++ ;
    }

    rc = SQLFreeHandle( SQL_HANDLE_STMT, hstmt ) ;
    CHECK_HANDLE( SQL_HANDLE_STMT, hstmt, rc ) ;

    rc = SQLEndTran( SQL_HANDLE_DBC, hdbc, SQL_COMMIT ) ;
    CHECK_HANDLE( SQL_HANDLE_DBC, hdbc, rc ) ;

    printf( ">Disconnecting .....\n" ) ;
    rc = SQLDisconnect( hdbc ) ;
    CHECK_HANDLE( SQL_HANDLE_DBC, hdbc, rc ) ;

    rc = SQLFreeHandle( SQL_HANDLE_DBC, hdbc ) ;
    CHECK_HANDLE( SQL_HANDLE_DBC, hdbc, rc ) ;

    rc = SQLFreeHandle( SQL_HANDLE_ENV,  henv ) ;
    if ( rc != SQL_SUCCESS )
        return( terminate( henv, rc ) ) ;

    return( SQL_SUCCESS ) ;

}                               /* end main */
