/*******************************************************************************
**                                                                        
** Source File Name = getfuncs.c  %I%                                      
**                                                                        
** Licensed Materials - Property of IBM                                   
**                                                                        
** (C) COPYRIGHT International Business Machines Corp. 1995, 1999
** All Rights Reserved.                                                   
**                                                                        
** US Government Users Restricted Rights - Use, duplication or            
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      
**                                                                        
**                                                                        
**    PURPOSE :                                                           
**    - Connect to a database
**    - Display connect information
**    - List all functions, and if they are supported
**                                                                        
** For more information about these samples see the README file.
**
** For more information on programming in CLI see the:
**     - "Building CLI Applications" section of the Application Building Guide, and the
**     - CLI Guide and Reference.
**
** For more information on the SQL language see the SQL Reference.
**
********************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sqlcli1.h>
#include "samputil.h"          /* Header file for CLI sample code */

/* For the Macintosh environment when generating 68K applications */
#ifdef DB268K
   /* Need to include ASLM for 68K applications */
   #include <LibraryManager.h>
#endif

/*--> SQLL1X06.SCRIPT */

typedef struct {
  SQLUSMALLINT id ;
  char * name ;
} functionInfo ;

functionInfo functions[] = {
  { SQL_API_SQLALLOCCONNECT,     "SQLALLOCCONNECT"     },
  { SQL_API_SQLALLOCENV,         "SQLALLOCENV"         },
  { SQL_API_SQLALLOCHANDLE,      "SQLALLOCHANDLE"      },
  { SQL_API_SQLALLOCSTMT,        "SQLALLOCSTMT"        },
  { SQL_API_SQLBINDCOL,          "SQLBINDCOL"          },
  { SQL_API_SQLBINDFILETOCOL,    "SQLBINDFILETOCOL"    },
  { SQL_API_SQLBINDFILETOPARAM,  "SQLBINDFILETOPARAM"  },
  { SQL_API_SQLBINDPARAM,        "SQLBINDPARAM"        },
  { SQL_API_SQLBINDPARAMETER,    "SQLBINDPARAMETER"    },
  { SQL_API_SQLBROWSECONNECT,    "SQLBROWSECONNECT"    },
  { SQL_API_SQLCANCEL,           "SQLCANCEL"           },
  { SQL_API_SQLCLOSECURSOR,      "SQLCLOSECURSOR"      },
  { SQL_API_SQLCOLATTRIBUTE,     "SQLCOLATTRIBUTE"     },
  { SQL_API_SQLCOLATTRIBUTES,    "SQLCOLATTRIBUTES"    },
  { SQL_API_SQLCOLUMNPRIVILEGES, "SQLCOLUMNPRIVILEGES" },
  { SQL_API_SQLCOLUMNS,          "SQLCOLUMNS"          },
  { SQL_API_SQLCONNECT,          "SQLCONNECT"          },
  { SQL_API_SQLCOPYDESC,         "SQLCOPYDESC"         },
  { SQL_API_SQLDATASOURCES,      "SQLDATASOURCES"      },
  { SQL_API_SQLDESCRIBECOL,      "SQLDESCRIBECOL"      },
  { SQL_API_SQLDESCRIBEPARAM,    "SQLDESCRIBEPARAM"    },
  { SQL_API_SQLDISCONNECT,       "SQLDISCONNECT"       },
  { SQL_API_SQLDRIVERCONNECT,    "SQLDRIVERCONNECT"    },
  { SQL_API_SQLENDTRAN,          "SQLENDTRAN"          },
  { SQL_API_SQLERROR,            "SQLERROR"            },
  { SQL_API_SQLEXECDIRECT,       "SQLEXECDIRECT"       },
  { SQL_API_SQLEXECUTE,          "SQLEXECUTE"          },
  { SQL_API_SQLEXTENDEDFETCH,    "SQLEXTENDEDFETCH"    },
  { SQL_API_SQLFETCH,            "SQLFETCH"            },
  { SQL_API_SQLFETCHSCROLL,      "SQLFETCHSCROLL"      },
  { SQL_API_SQLFOREIGNKEYS,      "SQLFOREIGNKEYS"      },
  { SQL_API_SQLFREECONNECT,      "SQLFREECONNECT"      },
  { SQL_API_SQLFREEENV,          "SQLFREEENV"          },
  { SQL_API_SQLFREEHANDLE,       "SQLFREEHANDLE"       },
  { SQL_API_SQLFREESTMT,         "SQLFREESTMT"         },
  { SQL_API_SQLGETCONNECTATTR,   "SQLGETCONNECTATTR"   },
  { SQL_API_SQLGETCONNECTOPTION, "SQLGETCONNECTOPTION" },
  { SQL_API_SQLGETCURSORNAME,    "SQLGETCURSORNAME"    },
  { SQL_API_SQLGETDATA,          "SQLGETDATA"          },
  { SQL_API_SQLGETDESCFIELD,     "SQLGETDESCFIELD"     },
  { SQL_API_SQLGETDESCREC,       "SQLGETDESCREC"       },
  { SQL_API_SQLGETDIAGFIELD,     "SQLGETDIAGFIELD"     },
  { SQL_API_SQLGETDIAGREC,       "SQLGETDIAGREC"       },
  { SQL_API_SQLGETENVATTR,       "SQLGETENVATTR"       },
  { SQL_API_SQLGETFUNCTIONS,     "SQLGETFUNCTIONS"     },
  { SQL_API_SQLGETINFO,          "SQLGETINFO"          },
  { SQL_API_SQLGETLENGTH,        "SQLGETLENGTH"        },
  { SQL_API_SQLGETPOSITION,      "SQLGETPOSITION"      },
  { SQL_API_SQLGETSQLCA,         "SQLGETSQLCA"         },
  { SQL_API_SQLGETSTMTATTR,      "SQLGETSTMTATTR"      },
  { SQL_API_SQLGETSTMTOPTION,    "SQLGETSTMTOPTION"    },
  { SQL_API_SQLGETSUBSTRING,     "SQLGETSUBSTRING"     },
  { SQL_API_SQLGETTYPEINFO,      "SQLGETTYPEINFO"      },
  { SQL_API_SQLMORERESULTS,      "SQLMORERESULTS"      },
  { SQL_API_SQLNATIVESQL,        "SQLNATIVESQL"        },
  { SQL_API_SQLNUMPARAMS,        "SQLNUMPARAMS"        },
  { SQL_API_SQLNUMRESULTCOLS,    "SQLNUMRESULTCOLS"    },
  { SQL_API_SQLPARAMDATA,        "SQLPARAMDATA"        },
  { SQL_API_SQLPARAMOPTIONS,     "SQLPARAMOPTIONS"     },
  { SQL_API_SQLPREPARE,          "SQLPREPARE"          },
  { SQL_API_SQLPRIMARYKEYS,      "SQLPRIMARYKEYS"      },
  { SQL_API_SQLPROCEDURECOLUMNS, "SQLPROCEDURECOLUMNS" },
  { SQL_API_SQLPROCEDURES,       "SQLPROCEDURES"       },
  { SQL_API_SQLPUTDATA,          "SQLPUTDATA"          },
  { SQL_API_SQLROWCOUNT,         "SQLROWCOUNT"         },
  { SQL_API_SQLSETCOLATTRIBUTES, "SQLSETCOLATTRIBUTES" },
  { SQL_API_SQLSETCONNECTATTR,   "SQLSETCONNECTATTR"   },
  { SQL_API_SQLSETCONNECTION,    "SQLSETCONNECTION"    },
  { SQL_API_SQLSETCONNECTOPTION, "SQLSETCONNECTOPTION" },
  { SQL_API_SQLSETCURSORNAME,    "SQLSETCURSORNAME"    },
  { SQL_API_SQLSETDESCFIELD,     "SQLSETDESCFIELD"     },
  { SQL_API_SQLSETDESCREC,       "SQLSETDESCREC"       },
  { SQL_API_SQLSETENVATTR,       "SQLSETENVATTR"       },
  { SQL_API_SQLSETPARAM,         "SQLSETPARAM"         },
  { SQL_API_SQLSETPOS,           "SQLSETPOS"           },
  { SQL_API_SQLSETSCROLLOPTIONS, "SQLSETSCROLLOPTIONS" },
  { SQL_API_SQLSETSTMTATTR,      "SQLSETSTMTATTR"      },
  { SQL_API_SQLSETSTMTOPTION,    "SQLSETSTMTOPTION"    },
  { SQL_API_SQLSPECIALCOLUMNS,   "SQLSPECIALCOLUMNS"   },
  { SQL_API_SQLSTATISTICS,       "SQLSTATISTICS"       },
  { SQL_API_SQLTABLEPRIVILEGES,  "SQLTABLEPRIVILEGES"  },
  { SQL_API_SQLTABLES,           "SQLTABLES"           },
  { SQL_API_SQLTRANSACT,         "SQLTRANSACT"         },
  { 0,                           ( char * ) 0          }
} ;

/*<-- */

/*
 Global Variables for user id and password.
 To keep samples simple, not a recommended practice.
*/
extern SQLCHAR server[SQL_MAX_DSN_LENGTH + 1] ;
extern SQLCHAR uid[MAX_UID_LENGTH + 1] ;
extern SQLCHAR pwd[MAX_PWD_LENGTH + 1] ;

/**  main  **/
int main( int argc, char * argv[] ) {

    SQLHANDLE henv, hdbc ;
    SQLRETURN rc ;

    SQLUSMALLINT supported ;

    int func_pos, side ;
    char b_line[4], e_line[3] ;

/* For the Macintosh environment when generating 68K applications */
#ifdef DB268K
   /* Before making any API calls for 68K environment, need to initialize the
      Library Manager */
   InitLibraryManager(0,kCurrentZone,kNormalMemory);
   atexit(CleanupLibraryManager);
#endif

    /* macro to initalize server, uid and pwd */
    INIT_UID_PWD ;

    /* allocate an environment handle */
    rc = SQLAllocHandle( SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv ) ;
    if ( rc != SQL_SUCCESS ) return( terminate( henv, rc ) ) ;

    /* allocate a connect handle, and connect */
    rc = DBconnect( henv, &hdbc ) ;
    if ( rc != SQL_SUCCESS ) return( terminate( henv, rc ) ) ;

    /* Print connection information */
    print_connect_info( hdbc ) ;

/*--> */

    func_pos = 0 ;
    side = 0 ;
    *b_line = '\0' ;
    *e_line = '\0' ;

    while ( functions[func_pos].id != 0 ) {
       SQLGetFunctions( hdbc,
                        functions[func_pos].id,
                        &supported
                      ) ;
       if ( supported )
          printf( "%s%-20s is supported%s",
                  b_line,
                  functions[func_pos].name,
                  e_line
                ) ;
       else
          printf( "%s%-20s is not supported%s",
                  b_line,
                  functions[func_pos].name,
                  e_line
                ) ;
       if ( side ) {
          *b_line = '\0' ;
          *e_line = '\0' ;
          side = 0 ;
       }
       else {
          strcpy( b_line, "   " ) ;
          strcpy( e_line, "\n" ) ;
          side = 1 ;
       }
       func_pos++ ;
    }

/*<-- */

    /*  Disconnect and free up CLI resources.  */

    /*  Commit the changes.  */
    rc = SQLEndTran( SQL_HANDLE_DBC, hdbc, SQL_COMMIT ) ;
    CHECK_HANDLE( SQL_HANDLE_DBC, hdbc, rc ) ;

    printf( "\n>Disconnecting .....\n" ) ;
    rc = SQLDisconnect( hdbc ) ;
    CHECK_HANDLE( SQL_HANDLE_DBC, hdbc, rc ) ;

    rc = SQLFreeHandle( SQL_HANDLE_DBC, hdbc ) ;
    CHECK_HANDLE( SQL_HANDLE_DBC, hdbc, rc ) ;

    rc = SQLFreeHandle( SQL_HANDLE_ENV,  henv ) ;
    if ( rc != SQL_SUCCESS ) return( terminate( henv, rc ) ) ;

    return( SQL_SUCCESS ) ;

}                                  /* end main */
