/*******************************************************************************
**                                                                        
** Source File Name = setpos.c  %I%                                      
**                                                                        
** Licensed Materials - Property of IBM                                   
**                                                                        
** (C) COPYRIGHT International Business Machines Corp. 1999
** All Rights Reserved.                                                   
**                                                                        
** US Government Users Restricted Rights - Use, duplication or            
** disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      
**                                                                        
**                                                                        
**    PURPOSE : To demonstrate the use of the SQLSetPos call, specifically the
**              use of the update, and delete options.               
**              This program accesses, and modifies the staff table in the
**              sample database. It assumes that the data in the table
**              has not been modified from its original source. Although it
**              doesn't leave it that way, so the table needs to be recreated
**              each time this program is executed (try exporting the data
**              to a file before starting).
**
**              As well, a unique key must be created on the table:
**          
**              db2 create unique index staffin on staff (id)
**
**                                                                        
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sqlcli1.h>
#include "samputil.h"          /* Header file for CLI sample code */

/* For the Macintosh environment when generating 68K applications */
#ifdef DB268K
   /* Need to include ASLM for 68K applications */
   #include <LibraryManager.h>
#endif

#define MAX_STMT_LEN 255

/*
 Global Variables for user id and password.
 To keep samples simple, not a recommended practice.
*/
extern SQLCHAR server[SQL_MAX_DSN_LENGTH + 1] ;
extern SQLCHAR uid[MAX_UID_LENGTH + 1] ;
extern SQLCHAR pwd[MAX_PWD_LENGTH + 1] ;

void printData(SQLINTEGER *id, SQLCHAR name[16][15],
               SQLINTEGER *dept, SQLCHAR job[16][10],
               SQLINTEGER *years, SQLDOUBLE *salary, SQLDOUBLE *comm);
void giveRaise(SQLDOUBLE *salary);

int main( int argc, char * argv[] ) {

    SQLHANDLE henv, hdbc, hselstmt, hinstmt ;
    SQLRETURN rc ;

    SQLCHAR * selstmt = ( SQLCHAR * )
       "SELECT id, name, dept, job, years, salary, comm FROM staff WHERE years > 6" ;


    SQLCHAR name[16][15], job[16][10];
    SQLINTEGER id[16], dept[16], years[16];
    SQLDOUBLE salary[16], comm[16];

    SQLINTEGER idInd[16], nameInd[16], deptInd[16], jobInd[16], yearsInd[16],
               salaryInd[16], commInd[16];


/* For the Macintosh environment when generating 68K applications */
#ifdef DB268K
   /* Before making any API calls for 68K environment, need to initialize the
      Library Manager */
   InitLibraryManager(0,kCurrentZone,kNormalMemory);
   atexit(CleanupLibraryManager);
#endif

    /* macro to initalize server, uid and pwd */
    INIT_UID_PWD ;

    /* allocate an environment handle */
    rc = SQLAllocHandle( SQL_HANDLE_ENV, SQL_NULL_HANDLE, &henv ) ;
    if ( rc != SQL_SUCCESS ) return( terminate( henv, rc ) ) ;

    /* allocate a connect handle, and connect */
    rc = DBconnect( henv, &hdbc ) ;
    if ( rc != SQL_SUCCESS ) return( terminate( henv, rc ) ) ;

    /* Allocate the Select statement handle */

    rc = SQLAllocHandle( SQL_HANDLE_STMT, hdbc, &hselstmt ) ;
    CHECK_HANDLE( SQL_HANDLE_DBC, hdbc, rc ) ;

    rc = SQLSetStmtAttr( hselstmt, SQL_ATTR_CURSOR_TYPE, (SQLPOINTER)SQL_CURSOR_KEYSET_DRIVEN,0);
    CHECK_HANDLE( SQL_HANDLE_DBC, hdbc, rc ) ;

    rc = SQLSetStmtAttr( hselstmt, SQL_ATTR_ROW_BIND_TYPE, SQL_BIND_BY_COLUMN, 0);
    CHECK_HANDLE( SQL_HANDLE_DBC, hdbc, rc ) ;

    rc = SQLSetStmtAttr( hselstmt, SQL_ATTR_ROW_ARRAY_SIZE, (SQLPOINTER)15, 0);
    CHECK_HANDLE( SQL_HANDLE_DBC, hdbc, rc ) ;  

    /* Execute the select */

    rc = SQLExecDirect( hselstmt, selstmt, SQL_NTS ) ;
    CHECK_HANDLE( SQL_HANDLE_STMT, hselstmt, rc ) ;

    /* Bind the columns to buffers */ 

    rc = SQLBindCol( hselstmt, 1, SQL_C_LONG, id, sizeof(id[0]), idInd);
    CHECK_HANDLE( SQL_HANDLE_STMT, hselstmt, rc );   
    rc = SQLBindCol( hselstmt, 2, SQL_C_CHAR, name, sizeof(name[0]), nameInd );
    CHECK_HANDLE( SQL_HANDLE_STMT, hselstmt, rc ) ;
    rc = SQLBindCol( hselstmt, 3, SQL_C_LONG, dept, sizeof(dept[0]), deptInd);
    CHECK_HANDLE( SQL_HANDLE_STMT, hselstmt, rc );
    rc = SQLBindCol( hselstmt, 4, SQL_C_CHAR, job, sizeof(job[0]), jobInd);
    CHECK_HANDLE( SQL_HANDLE_STMT, hselstmt, rc );
    rc = SQLBindCol( hselstmt, 5, SQL_C_LONG, years, sizeof(years[0]), yearsInd ) ;
    CHECK_HANDLE( SQL_HANDLE_STMT, hselstmt, rc ) ;
    rc = SQLBindCol( hselstmt, 6, SQL_C_DOUBLE, salary, sizeof(salary[0]), salaryInd ) ;
    CHECK_HANDLE( SQL_HANDLE_STMT, hselstmt, rc ) ;
    rc = SQLBindCol( hselstmt, 7, SQL_C_DOUBLE, comm, sizeof(comm[0]), commInd ) ;
    CHECK_HANDLE( SQL_HANDLE_STMT, hselstmt, rc ) ;

    /* Fetch and display the data */

    printf("Fetching modified data\ngiving a raise, and updating\n");

    while ((rc=SQLFetchScroll(hselstmt, SQL_FETCH_NEXT, 0) == SQL_SUCCESS))
    {
        printf("--> here's the data\n");
        printData(id,name,dept,job,years,salary,comm);
        printf("--> Give everyone a raise\n");
        giveRaise(salary);
        rc = SQLSetPos(hselstmt, 0, SQL_UPDATE, SQL_LOCK_NO_CHANGE);
        CHECK_HANDLE( SQL_HANDLE_STMT, hselstmt, rc ) ;
    }

    if ( rc != SQL_NO_DATA_FOUND )
        CHECK_HANDLE( SQL_HANDLE_STMT, hselstmt, rc ) ;

    /* commit the changes */ 

    printf("Commiting the update\nand freeing the statement\n");

    rc = SQLEndTran( SQL_HANDLE_DBC, hdbc, SQL_COMMIT ) ;
    CHECK_HANDLE( SQL_HANDLE_DBC, hdbc, rc ) ;

    rc = SQLFreeHandle( SQL_HANDLE_STMT, hselstmt);
    CHECK_HANDLE( SQL_HANDLE_STMT, hselstmt, rc);

 /* Allocate the Select statement handle */

    rc = SQLAllocHandle( SQL_HANDLE_STMT, hdbc, &hselstmt ) ;
    CHECK_HANDLE( SQL_HANDLE_DBC, hdbc, rc ) ;

    rc = SQLSetStmtAttr( hselstmt, SQL_ATTR_CURSOR_TYPE, (SQLPOINTER)SQL_CURSOR_KEYSET_DRIVEN,0);
    CHECK_HANDLE( SQL_HANDLE_DBC, hdbc, rc ) ;

    rc = SQLSetStmtAttr( hselstmt, SQL_ATTR_ROW_BIND_TYPE, SQL_BIND_BY_COLUMN, 0);
    CHECK_HANDLE( SQL_HANDLE_DBC, hdbc, rc ) ;

    rc = SQLSetStmtAttr( hselstmt, SQL_ATTR_ROW_ARRAY_SIZE, (SQLPOINTER)15, 0);
    CHECK_HANDLE( SQL_HANDLE_DBC, hdbc, rc ) ;  

    /* Execute the select */

    rc = SQLExecDirect( hselstmt, selstmt, SQL_NTS ) ;
    CHECK_HANDLE( SQL_HANDLE_STMT, hselstmt, rc ) ;

    /* Bind the columns to buffers */ 

    rc = SQLBindCol( hselstmt, 1, SQL_C_LONG, id, sizeof(id[0]), idInd);
    CHECK_HANDLE( SQL_HANDLE_STMT, hselstmt, rc );   
    rc = SQLBindCol( hselstmt, 2, SQL_C_CHAR, name, sizeof(name[0]), nameInd );
    CHECK_HANDLE( SQL_HANDLE_STMT, hselstmt, rc ) ;
    rc = SQLBindCol( hselstmt, 3, SQL_C_LONG, dept, sizeof(dept[0]), deptInd);
    CHECK_HANDLE( SQL_HANDLE_STMT, hselstmt, rc );
    rc = SQLBindCol( hselstmt, 4, SQL_C_CHAR, job, sizeof(job[0]), jobInd);
    CHECK_HANDLE( SQL_HANDLE_STMT, hselstmt, rc );
    rc = SQLBindCol( hselstmt, 5, SQL_C_LONG, years, sizeof(years[0]), yearsInd ) ;
    CHECK_HANDLE( SQL_HANDLE_STMT, hselstmt, rc ) ;
    rc = SQLBindCol( hselstmt, 6, SQL_C_DOUBLE, salary, sizeof(salary[0]), salaryInd ) ;
    CHECK_HANDLE( SQL_HANDLE_STMT, hselstmt, rc ) ;
    rc = SQLBindCol( hselstmt, 7, SQL_C_DOUBLE, comm, sizeof(comm[0]), commInd ) ;
    CHECK_HANDLE( SQL_HANDLE_STMT, hselstmt, rc ) ;

    printf("Show all the data again.\nThis time we'll delete it all instead\n");
    while ((rc=SQLFetchScroll(hselstmt, SQL_FETCH_NEXT, 0) == SQL_SUCCESS))
    {
        printData(id,name,dept,job,years,salary,comm);
        rc = SQLSetPos(hselstmt, 0, SQL_DELETE, SQL_LOCK_NO_CHANGE);
        CHECK_HANDLE( SQL_HANDLE_STMT, hselstmt, rc ) ;
    }

    if ( rc != SQL_NO_DATA_FOUND )
        CHECK_HANDLE( SQL_HANDLE_STMT, hselstmt, rc ) ;

    /* Free select statement */
    printf("Commiting, and Freeing select\n");

    rc = SQLEndTran( SQL_HANDLE_DBC, hdbc, SQL_COMMIT ) ;
    CHECK_HANDLE( SQL_HANDLE_DBC, hdbc, rc ) ;

    rc = SQLFreeHandle( SQL_HANDLE_STMT, hselstmt);
    CHECK_HANDLE( SQL_HANDLE_STMT, hselstmt, rc);

    printf( "\n>Disconnecting .....\n" ) ;
    rc = SQLDisconnect( hdbc ) ;
    CHECK_HANDLE( SQL_HANDLE_DBC, hdbc, rc ) ;

    rc = SQLFreeHandle( SQL_HANDLE_DBC, hdbc ) ;
    CHECK_HANDLE( SQL_HANDLE_DBC, hdbc, rc ) ;

    rc = SQLFreeHandle( SQL_HANDLE_ENV,  henv ) ;
    if ( rc != SQL_SUCCESS ) return( terminate( henv, rc ) ) ;

    return( SQL_SUCCESS ) ;
}

void printData(SQLINTEGER *id, SQLCHAR name[16][15], 
               SQLINTEGER *dept, SQLCHAR job[16][10],
               SQLINTEGER *years, SQLDOUBLE *salary, SQLDOUBLE *comm)
{
    int i;
    for(i=0;i<15;i++){
        printf("%d ",id[i]);
        printf("%s ",name[i]);
        printf("%d ",dept[i]);
        printf("%s ",job[i]);
        printf("%d ",years[i]);
        printf("%f ",salary[i]);
        printf("%f\n",comm[i]);
    }
}

void giveRaise(SQLDOUBLE *salary)
{
    int i;
    for(i=0;i<15;i++){
        salary[i]=(salary[i]*(double)1.5);
    }
}
