//
//  Source File Name: JobUpdat.java  %I%
//
//  Licensed Materials -- Property of IBM
//
//  (c) Copyright International Business Machines Corporation, 1999.
//      All Rights Reserved.
//
//  US Government Users Restricted Rights -
//  Use, duplication or disclosure restricted by
//  GSA ADP Schedule Contract with IBM Corp.
//
//  Example of using a prepared statement and and SQL statement to run through
//    and update a table in a database.  This program functions the same as the
//    getcurs.c and setcurs.c programs.  However, in this version JDBC does not
//    support updatable queries.  Therefore the program had to be written using
//    a prepared statement.

// For more information about this sample, refer to the README file.

// For more information on programming in Java, refer to the 
// "Programming in Java" section of the Application Development Guide.

// For more information on building and running Java programs for DB2,
// refer to the "Building Java Applets and Applications" section of the 
// Application Building Guide.

// For more information on the SQL language, refer to the SQL Reference.

import java.io.*;
import java.lang.*;
import java.sql.*;

class JobUpdat {

   static {
      try {
         // register the driver with DriverManager
         // The newInstance() call is needed for the sample to work with
         // JDK 1.1.1 on OS/2, where the Class.forName() method does not
         // run the static initializer. For other JDKs, the newInstance
         // call can be omitted.
         Class.forName("COM.ibm.db2.jdbc.app.DB2Driver").newInstance();
      } catch (Exception e) {
         e.printStackTrace();
      }
   }

   public static void main (String[] argv) {
      Connection con = null;

      try {
         // connect with id/password
         if (argv.length == 3)
         {  // connect using command line arguments
            con = Tools.DBConnect(argv[0], argv[1], argv[2]); 
         }
         else
         {  // prompt user for database name, user ID, password
            con = Tools.DBConnect(); 
         }
         con.setAutoCommit (false);

         showAndUpdateClerks (con);

         System.out.println ("Committing Transaction");
         con.commit();
         //disconnect from connection made to server
         System.out.print ("\n>Disconnecting...\n");
         con.close();

      } catch (Exception e) { e.printStackTrace(); }
   }

   //
   // showAndUpdateClerks
   //    - shows user current clerks and prompts for job change
   //    - then job changes are made to the database
   //

   public static void showAndUpdateClerks (Connection con) {
     try {
         Statement stmt = con.createStatement();
         PreparedStatement pstmt = con.prepareStatement ("UPDATE staff set job = ? where id = ?");
         ResultSet rs = stmt.executeQuery ("SELECT id, name, job FROM staff WHERE job = 'Clerk'");
         String cursor = rs.getCursorName();

         System.out.println ("Job Change for all clerks");
         String newJob = new String("");
         String job = new String("");
         String name = new String("");
         int id = 0;

         // runs the result set displaying current employees that are clerks
         // if user enters a new job the prepared statement is executed,
         // which updates the JOB column of the STAFF table
         while (rs.next()) {
            id = rs.getInt (1);
            name = rs.getString (2);
            job = rs.getString (3);
            System.out.println ("Name: " + Tools.padLength(name, 13) + " Job: " + job);
            System.out.println ("Enter new job or return to continue");
            newJob = Tools.readString();
            if (!(newJob.equals(""))) {
              pstmt.setString (1, newJob);
              pstmt.setInt (2, id);
              pstmt.executeUpdate();
            }
         }
         rs.close();
         pstmt.close();
         stmt.close();
     } catch (Exception e) { e.printStackTrace(); }
   }
}
